@@ -3,7 +3,7 @@ package YATT::Toplevel::CGI;
 use strict;
 use warnings FATAL => qw(all);
 
-use Exporter qw(import);
+BEGIN {require Exporter; *import = \&Exporter::import}
 
 use base qw(File::Spec);
 use File::Basename;
@@ -29,17 +29,23 @@ use YATT::Exception;
 use base qw(YATT::Class::Configurable);
 use YATT::Types -base => __PACKAGE__
   , [Config => [qw(^cf_registry
+		   cf_driver
 		   cf_docs cf_tmpl
 		   cf_charset
+		   cf_debug_allowed_ip
 		   cf_translator_param
 		   cf_user_config
 		   cf_no_header
 		   cf_allow_unknown_config
 		   cf_auto_reload
+		   cf_no_chdir
+		   cf_rlimit
+		   cf_use_session
 		 )
 		, ['^cf_app_prefix' => 'YATT']
 		, ['^cf_find_root_upward' => 2]
-	       ]];
+	       ]]
+  , qw(:export_alias);
 
 Config->define(create => \&create_toplevel);
 
@@ -48,18 +54,24 @@ Config->define(create => \&create_toplevel);
 use vars map {'$'.$_} our @env_vars
   = qw(DOCUMENT_ROOT
        PATH_INFO
-       SCRIPT_FILENAME
+       PATH_TRANSLATED
+       REDIRECT_REDIRECT_STATUS
        REDIRECT_STATUS
-       PATH_TRANSLATED);
-our @EXPORT = (qw(&use_env_vars
+       REDIRECT_URL
+       REQUEST_URI
+       SCRIPT_FILENAME
+     );
+push our @EXPORT, (qw(&use_env_vars
 		  &rootname
 		  &capture
+		  &new_config
 		), map {'*'.$_} our @env_vars);
 
 our Config $CONFIG;
-our ($CGI, $SESSION, %COOKIE, %HEADER);
-sub rc_global () { qw(CONFIG CGI SESSION HEADER COOKIE) }
-our @EXPORT_OK = (@EXPORT, map {'*'.$_} rc_global);
+our ($CGI, $SESSION, %COOKIE, %HEADER, $RANDOM_LIST, $RANDOM_INDEX);
+sub rc_global () { qw(CONFIG CGI SESSION HEADER COOKIE
+		      RANDOM_LIST RANDOM_INDEX) }
+push our @EXPORT_OK, (@EXPORT, map {'*'.$_} rc_global);
 
 sub ROOT_CONFIG () {'.htyattroot'}
 
@@ -84,46 +96,88 @@ sub run_cgi {
 
   local $CONFIG = my Config $config = $pack->new_config(shift);
 
-  my ($root, $file, $error);
+  my ($root, $file, $error, $param);
   if (catch {
-    ($pack, $root, $cgi, $file) = $pack->prepare_dispatch($cgi, $config);
-    } \ $error or catch {
-      $pack->dispatch($root, $cgi, $file);
-    } \ $error and not is_normal_end($error)) {
+    ($pack, $root, $cgi, $file, $param)
+      = $pack->prepare_dispatch($cgi, $config);
+  } \ $error) {
+    $pack->dispatch_error($root, $error
+			  , {phase => 'prepare', target => $file});
+  } else {
+    $pack->run_retry_max(3, $root, $file, $cgi, $param);
+  }
+}
+
+sub run_retry_max {
+  my ($pack, $max, $root_or_config, $file, $cgi, @param) = @_;
+  my $root = do {
+    if (UNIVERSAL::isa($root_or_config, Config)) {
+      my Config $config = $root_or_config;
+      $config->{cf_registry}
+    } else {
+      $root_or_config;
+    }
+  };
+  my $rc = catch {
+    $pack->dispatch($root, $cgi, $file, @param);
+  } \ my $error;
+  if ($rc) {
+    my ($i) = (0);
+    while ($rc and ($file, $cgi) = can_retry($error)) {
+      if ($i++ > $max) {
+	$pack->dispatch_error($root, $error
+			      , {phase => 'retry', target => $file});
+	undef $error;
+	last;
+      }
+      $rc = catch {
+	$pack->dispatch($root, $cgi, $file);
+      } \ $error;
+    }
+  }
+  if ($rc and not is_normal_end($error)) {
     $pack->dispatch_error($root, $error
 			  , {phase => 'action', target => $file});
   }
 }
 
 sub create_toplevel {
-  (my Config $config, my ($dir)) = splice @_, 0, 2;
-
-  $dir ||= '.';
-
+  my $pack = shift;
+  my Config $config = $pack->new_config(shift);
   $config->configure(@_) if @_;
-
-  $config->try_load_config($dir);
+  my $dir = $config->{cf_docs} ||= '.';
+  $pack->can('try_load_config')->($config, $dir);
+  my $instpkg = $pack->get_instpkg($config);
 
   my @loader = (DIR => $config->{cf_docs});
-
   push @loader, LIB => $config->{cf_tmpl} if $config->{cf_tmpl};
 
-  $config->{cf_registry} = $config->new_translator
+  my $trans = $config->{cf_registry} = $instpkg->new_translator
     (\@loader, $config->translator_param);
 
-  $config;
+  ($instpkg, $trans, $config);
 }
 
+#
+# XXX: should be: create_toplevel_from_cgi($cgi, $config)
+# => ($instpkg, $trans, $config, $cgi, $file, $param);
+# since $config->{cf_registry} points $translator.
+#
 sub prepare_dispatch {
   (my ($pack, $cgi), my Config $config) = @_;
-  my ($rootdir, $file, $loader) = do {
+  my ($rootdir, $file, $loader, $param) = do {
     if (not $config->{cf_registry} and $config->{cf_docs}) {
       # $config->try_load_config($config->{cf_docs});
       ($config->{cf_docs}, $cgi->path_info
        , [DIR => $config->{cf_docs}]);
-    } elsif ($REDIRECT_STATUS and $PATH_TRANSLATED) {
-      ($pack->param_for_redirect($PATH_TRANSLATED
-				 , $SCRIPT_FILENAME || $0, $config));
+    } elsif ($REDIRECT_STATUS) {
+      # 404 Not found handling
+      my $target = $PATH_TRANSLATED || $DOCUMENT_ROOT . $REDIRECT_URL;
+      # This ensures .htyattroot is loaded.
+      ($pack->param_for_redirect($target
+				 , $SCRIPT_FILENAME || $0, $config
+				 , $REDIRECT_STATUS == 404
+				));
     } elsif ($PATH_INFO and $SCRIPT_FILENAME) {
       (untaint_any(dirname($SCRIPT_FILENAME))
        , untaint_any($PATH_INFO)
@@ -147,6 +201,7 @@ END
 
   unless ($PATH_INFO) {
     if ($PATH_TRANSLATED) {
+      # XXX: ミス時に効率悪い。substr して eq に書き直すべき。
       if (index($PATH_TRANSLATED, $rootdir) == 0) {
 	$PATH_INFO = substr($PATH_TRANSLATED, length($rootdir));
       }
@@ -155,19 +210,42 @@ END
 
   $cgi->charset($config->{cf_charset} || 'utf-8');
 
-  my $instpkg = $config->app_prefix || 'main';
-  {
-    $pack->add_isa($instpkg, $pack);
-    foreach my $name ($pack->rc_global) {
-      *{globref($instpkg, $name)} = *{globref(MY, $name)};
-    }
-  }
+  my $instpkg = $pack->get_instpkg($config);
 
   my $root = $config->{cf_registry} ||= $instpkg->new_translator
     ($loader, $config->translator_param
      , debug_translator => $ENV{DEBUG});
 
-  ($instpkg, $root, $cgi, $file);
+  $instpkg->set_random_list;
+
+  $instpkg->force_parameter_convention($cgi); # XXX: unless $config->{...}
+
+  ($instpkg, $root, $cgi, $file, $param);
+}
+
+our $PARAM_CONVENTION = qr{^[\w:\-]};
+
+sub force_parameter_convention {
+  my ($pack, $cgi) = @_;
+  my @deleted;
+  foreach my $name ($cgi->param) {
+    next if $name =~ $PARAM_CONVENTION;
+    push @deleted, [$name => $cgi->param($name)];
+    $cgi->delete($name);
+  }
+  @deleted;
+}
+
+*get_instpkg = \&prepare_export;
+sub prepare_export {
+  my ($pack, $config, $instpkg) = @_;
+  $instpkg ||= $config && $config->app_prefix || 'main';
+
+  $pack->add_isa($instpkg, $pack);
+  foreach my $name ($pack->rc_global) {
+    *{globref($instpkg, $name)} = *{globref(MY, $name)};
+  }
+  $instpkg
 }
 
 sub run_template {
@@ -191,25 +269,62 @@ sub bye {
 			    , caller => [caller], @_);
 }
 
+sub raise_retry {
+  my ($pack, $file, $cgi, @param) = @_;
+  die $pack->Exception->new(error => '', retry => [$file, $cgi, @param]
+			    , caller => [caller])
+}
+
 sub dispatch {
   my ($top, $root, $cgi, $file, @param) = @_;
   &YATT::break_dispatch;
 
+  $root->mark_load_failure;
+
   local $CGI = $cgi;
   local ($SESSION, %COOKIE, %HEADER);
-  my ($renderer, $pkg);
+  if ($CONFIG->{cf_use_session}) {
+    $SESSION = $top->new_session($cgi);
+  }
+  my @elpath = $root->parse_elempath($top->canonicalize_html_filename($file));
+  my ($found, $renderer, $pkg, $widget);
 
   if (catch {
-    ($renderer, $pkg) = $root->get_handler_to
-      (render => $top->canonicalize_html_filename($file));
+    $found = ($renderer, $pkg, $widget)
+      = $root->lookup_handler_to(render => @elpath);
   } \ my $error) {
     $top->dispatch_error($root, $error
 			 , {phase => 'get_handler', target => $file});
+  } elsif (not $found) {
+    # XXX: これも。
+    $top->dispatch_not_found($root, $file, @param);
+  } elsif (not defined $renderer) {
+    $top->dispatch_error($root, "Can't compile: $file"
+			 , {phase => 'get_handler', target => $file});
   } else {
-    $top->dispatch_action($root, $renderer, $pkg, @param);
+    unless ($CONFIG->{cf_no_chdir}) {
+      # XXX: これもエラー処理を
+      my $dir = untaint_any(dirname($widget->filename));
+      chdir($dir);
+    }
+    if (not defined $param[0] and $widget->public) {
+      $param[0] = $widget->reorder_cgi_params($cgi);
+    }
+    if (my $handler = $pkg->can('dispatch_action')) {
+      $handler->($top, $root, $renderer, $pkg, @param);
+    } else {
+      $top->dispatch_action($root, $renderer, $pkg, @param);
+    }
   }
 }
 
+sub dispatch_not_found {
+  my ($top, $root, $file) = @_;
+  my $ERR = \*STDOUT;
+
+  print $ERR "\n\nNot found: $file";
+}
+
 # XXX: もう少し改善を。
 sub dispatch_error {
   my ($top, $root, $error, $info) = @_;
@@ -223,7 +338,10 @@ sub dispatch_error {
   } \ my $load_error) {
     print $ERR "\n\nload_error($load_error), original_error=($error)";
   } elsif (not $found) {
-    print $ERR "\n\n$error";
+    print $ERR $CGI ? $CGI->header : "\n\n";
+    print $ERR $error;
+    $top->printenv_html($info, id => 'error_info') if $info;
+    $top->printenv_html;
   } elsif (catch {
     $html = capture {$renderer->($pkg, [$error, $info])};
   } \ my Exception $error2) {
@@ -247,11 +365,14 @@ sub dispatch_error {
 sub dispatch_action {
   my ($top, $root, $action, $pkg, @param) = @_;
   &YATT::break_handler;
-  # XXX: $CONFIG->{cf_no_header}
-  my $html = capture { $action->($pkg, @param) };
-  # XXX: SESSION, COOKIE, HEADER...
-  print $CGI->header;
-  print $html;
+  if ($CONFIG && $CONFIG->{cf_no_header}) {
+    $action->($pkg, @param);
+  } else {
+    my $html = capture { $action->($pkg, @param) };
+    # XXX: SESSION, COOKIE, HEADER...
+    print $SESSION ? $SESSION->header : $CGI->header;
+    print $html;
+  }
   $top->bye;
 }
 
@@ -259,7 +380,25 @@ sub plain_error {
   my ($pack, $cgi, $message) = @_;
   print $cgi->header if $cgi;
   print $message;
-  exit ($cgi ? 0 : 1);
+  $pack->printenv_html;
+  $pack->plain_exit($cgi ? 0 : 1);
+}
+
+sub plain_exit {
+  my ($pack, $exit_code) = @_;
+  exit $exit_code;
+}
+
+sub printenv_html {
+  my ($pack, $env, %opts) = @_;
+  $opts{id} ||= 'printenv';
+  my $ERR = \*STDOUT;
+  $env ||= \%ENV;
+  print $ERR "<table id='$opts{id}'>\n";
+  foreach my $k (sort keys %$env) {
+    print $ERR "<tr><td>", $k, "</td><td>", $env->{$k}, "</td></tr>\n";
+  }
+  print $ERR "</table>\n";
 }
 
 #========================================
@@ -320,19 +459,40 @@ sub try_load_config {
 
   return unless -r $file;
 
+  # XXX: configure_by_file
   my @param = do {
     require YATT::XHF;
     my $parser = new YATT::XHF(filename => $file);
     $parser->read_as('pairlist');
   };
+  $config->heavy_configure(@param);
+}
 
-  $config->classify_config_param(@param);
+sub trim_trailing_pathinfo {
+  my ($pack, $strref, @prefix) = @_;
+  @prefix = ('') unless @prefix;
+  my @dirs = $pack->splitdir($$strref);
+  my @found;
+  while (@dirs and -e join("/", @prefix, @found, $dirs[0])) {
+    push @found, shift @dirs;
+  }
+  $$strref = join("/", @found);
+  return unless @dirs;
+  join("/", @dirs);
 }
 
 sub param_for_redirect {
-  (my ($pack, $path_translated, $script_filename), my Config $cfobj) = @_;
+  (my ($pack, $path_translated, $script_filename)
+   , my Config $cfobj, my $not_found) = @_;
   my $driver = untaint_any(rootname($script_filename));
 
+  my @params;
+  if (not $not_found and not -e $path_translated) {
+    # not_found でもないのに、 path_translated が not exists であるケース
+    # == trailing path_info が有るケース。
+    push @params, $pack->trim_trailing_pathinfo(\$path_translated);
+  }
+
   # This should set $cfobj->{cf_docs}
   unless ($cfobj->{cf_registry}) {
     # .htyattroot の読み込みは、registry 作成前の一度で十分。
@@ -345,7 +505,7 @@ sub param_for_redirect {
   my @loader = (DIR => $cfobj->{cf_docs}
 		, $pack->tmpl_for_driver($driver));
 
-  return ($cfobj->{cf_docs}, $target, \@loader);
+  return ($cfobj->{cf_docs}, $target, \@loader, @params ? \@params : ());
 }
 
 #========================================
@@ -375,9 +535,31 @@ sub new_cgi {
   }
 }
 
+sub new_session {
+  my ($toplevel, $cgi) = @_;
+  require CGI::Session;
+  my ($dsn, @opts) = do {
+    if (ref $CONFIG->{cf_use_session}) {
+      @{$CONFIG->{cf_use_session}}
+    } else {
+      $CONFIG->{cf_use_session}
+    }
+  };
+  CGI::Session->new($dsn, $cgi, @opts);
+}
+
+sub entity_session {
+  my ($pack, $name) = @_;
+  $SESSION->param($name);
+}
+
+sub entity_save_session {
+  $SESSION->save_param;
+}
+
 sub new_config {
   my $pack = shift;
-  my $config = @_ == 1 ? shift : \@_;
+  my Config $config = @_ == 1 ? shift : \@_;
   return $config if defined $config
     and ref $config and UNIVERSAL::isa($config, Config);
 
@@ -385,9 +567,11 @@ sub new_config {
     $pack = $pack->Config;
   }
 
-  $pack->new(do {
+  $config = $pack->new(do {
     unless (defined $config) {
       ()
+    } elsif (not ref $config) {
+      (docs => $config)
     } elsif (ref $config eq 'ARRAY') {
       @$config
     } elsif (ref $config eq 'HASH') {
@@ -398,31 +582,59 @@ Invalid configuration parameter: $config
 END
     }
   });
+
+  $config->{cf_driver} = $0;
+
+  $config;
 }
 
-sub classify_config_param {
+sub heavy_configure {
   my Config $config = shift;
   my $config_keys = $config->fields_hash;
   my $trans_keys = $config->load_type('Translator')->fields_hash_of_class;
   my (@mine, @trans, @unknown);
   while (my ($name, $value) = splice @_, 0, 2) {
-    if ($config_keys->{"cf_$name"}) {
+    my $mine = $config_keys->{"cf_$name"};
+    if ($mine) {
       push @mine, $name, $value;
     }
     if ($trans_keys->{"cf_$name"}) {
       push @trans, [$name, $value];
-    } else {
+    } elsif (not $mine) {
       push @unknown, [$name, $value];
     }
   }
   $config->configure(@mine) if @mine;
+  foreach my $name ($config->configkeys) {
+    if ($trans_keys->{"cf_$name"}
+	and defined (my $value = $config->{"cf_$name"})) {
+      push @trans, [$name, $value];
+    }
+  }
   $config->{cf_translator_param}{$_->[0]} = $_->[1] for @trans;
-  if (@unknown and $config->{cf_allow_unknown_config}) {
+  if (@unknown) {
+    unless ($config->{cf_allow_unknown_config}) {
+      croak "Unknown config opts: "
+	. join(", ", map {join("=", @$_)} @unknown);
+    }
     $config->{cf_user_config}{$_->[0]} = $_->[1] for @unknown;
   }
   $config;
 }
 
+sub configure_rlimit {
+  (my Config $config, my $rlimit_hash) = @_;
+  my $class = 'YATT::Util::RLimit';
+  eval qq{require $class} or die $@;
+  while (my ($rsrc, $limit) = each %$rlimit_hash) {
+    if (my $sub = $class->can("rlimit_" . $rsrc)) {
+      $sub->($limit);
+    } else {
+      $class->can('rlimit')->("RLIMIT_" . uc($rsrc), $limit);
+    }
+  }
+}
+
 sub extract_cgi_params {
   my ($pack, $cgi) = @_;
   my %param;
@@ -459,18 +671,128 @@ sub use_env_vars {
 
 #========================================
 
+sub set_random_list {
+  my ($this, $random) = @_;
+  if (defined $random) {
+    $RANDOM_LIST = ref $random ? $random : [split " ", $random];
+    $RANDOM_INDEX = 0;
+  } else {
+    undef $RANDOM_LIST;
+    undef $RANDOM_INDEX;
+  }
+}
+
+sub entity_rand {
+  my ($this, $scalar) = @_;
+  $scalar ||= 1;
+  if ($RANDOM_LIST) {
+    my $val = $RANDOM_LIST->[$RANDOM_INDEX++ % @$RANDOM_LIST];
+    $val * $scalar;
+  } else {
+    rand $scalar;
+  }
+}
+
+sub entity_randomize {
+  my ($this) = shift;
+  my $sub = $this->can('entity_rand');
+  my @result;
+  push @result, splice @_, $sub->($this, scalar @_), 1 while @_;
+  wantarray ? @result : \@result;
+}
+
+sub entity_breakpoint {
+  &YATT::breakpoint();
+}
+
+sub entity_concat {
+  my $this = shift;
+  join '', @_;
+}
+
+sub entity_join {
+  my ($this, $sep) = splice @_, 0, 2;
+  join $sep, grep {defined $_ && $_ ne ''} @_;
+}
+
+sub entity_format {
+  my ($this, $format) = (shift, shift);
+  sprintf $format, @_;
+}
+
+sub entity_is_debug_allowed {
+  my ($this) = @_;
+  unless (defined $CGI->{'.allow_debug'}) {
+    $CGI->{'.allow_debug'} = $this->is_debug_allowed($CGI->remote_addr);
+  }
+  $CGI->{'.allow_debug'};
+}
+
+sub is_debug_allowed {
+  my ($this, $ip) = @_;
+  my $pat = $$CONFIG{cf_debug_allowed_ip};
+  unless (defined $pat) {
+    $pat = $$CONFIG{cf_debug_allowed_ip} = $this->load_htdebug;
+  } elsif (ref $pat) {
+    $pat = $$CONFIG{cf_debug_allowed_ip} = qr{@{[join "|", map {"^$_"} @$pat]}};
+  } elsif ($pat eq '') {
+    return 0
+  }
+  $ip =~ $pat;
+}
+
+sub load_htdebug {
+  my ($this) = @_;
+  my $dir = untaint_any(dirname($CONFIG->{cf_driver}));
+  my $fn = "$dir/.htdebug";
+  return '' unless -r $fn;
+  open my $fh, '<', $fn or die "Can't open $fn: $!";
+  local $_;
+  my @pat;
+  while (<$fh>) {
+    chomp;
+    s/\#.*//;
+    next unless /\S/;
+    push @pat, '^'.quotemeta($_);
+  }
+  qr{@{[join "|", @pat]}};
+}
+
+sub entity_CGI { $CGI }
+
+sub entity_remote_addr {
+  $CGI->remote_addr
+}
+
+#========================================
+
 sub entity_param {
   my ($this) = shift;
   $CGI->param(@_);
 }
 
+#
+# For &HTML(); shortcut.
+# To use this, special_entities should have 'HTML'.
+#
+sub entity_HTML {
+  my $this = shift;
+  \ join "", grep {defined $_} @_;
+}
+
+sub entity_dump {
+  shift;
+  YATT::Util::terse_dump(@_);
+}
+
 #========================================
 
 sub canonicalize_html_filename {
   my $pack = shift;
   $_[0] .= "index" if $_[0] =~ m{/$};
-  $_[0] =~ s{\.(html?|yatt?)$}{};
-  $_[0]
+  my $copy = shift;
+  $copy =~ s{\.(y?html?|yatt?)$}{};
+  $copy;
 }
 
 sub widget_path_in {
@@ -495,6 +817,7 @@ sub YATT::Toplevel::CGI::Config::translator_param {
       , $config->{cf_translator_param})
 }
 
+
 #========================================
 package YATT::Toplevel::CGI::Batch; use YATT::Inc;
 use base qw(YATT::Toplevel::CGI);
